# TwitterAPIKit

Swift library for the Twitter API v1 and v2 (Work in progress).

---

Unfortunately, I couldn't find any active Twitter API library for Swift at the moment.

So, I decided to create one.

## Policy

- No dependencies

## Example

```swift
    let consumerKey = ""
    let consumerSecret = ""
    let oauthToken = ""
    let oauthTokenSecret = ""

    let client = TwitterAPIKit(
        consumerKey: consumerKey,
        consumerSecret: consumerSecret,
        oauthToken: oauthToken,
        oauthTokenSecret: oauthTokenSecret
    )

    client.v1.getShowStatus(.init(id: "status id")) { result in

        // Use result utils
        do {
            let success = try result.serialize().get() // JSONSerialization
            // Or let success = try result.decode(YourModel.self).get() // JSONDecoder
            print(success.rateLimit)
            print(success.data)
        } catch let error {
            print(error)
        }
        // ----

        // Or

        switch result {
        case let .success((data, rateLimit, httpURLResponse)):
            print("--- Success ---")
            print("Rate Limit", rateLimit)

            // parse json
            let obj = JSONSerialization.jsonObject(with: data, options: [])

            // Or decode
            let decoder = JSONDecoder()
            let product = try decoder.decode(YourModel.self, from: data)

        case .failure(let error): // TwitterAPIKitError
            print("--- Error ---")
            if let data = error.data {
                print("Body:", String(data: data, encoding: .utf8)!)
            }
            if let rateLimit = error.rateLimit {
                print("Rate Limit", rateLimit)
            }
        }
    }
```

### Custom Request class

The class of each request can be inherited to create subclasses. This is why it is declared as an open class instead of a struct.

This is intended so that when new parameters are added due to changes in the Twitter API, you can handle them yourself without waiting for the library to be updated.

```swift
// example
class CustomListsListRequestV1: GetListsListRequestV1 {

    let custom: String

    override var parameters: [String: Any] {
        var p = super.parameters
        p["custom"] = custom
        return p
    }

    init(custom: String, user: TwitterUserIdentifierV1, reverse: Bool? = .none) {
        self.custom = custom
        super.init(user: user, reverse: reverse)
    }
}
```

It is also possible to create an encapsulated custom request class.

```swift
class CapsuledListsListRequestV1: GetListsListRequestV1 {
    init() {
        super.init(user: .userID("100"), reverse: true)
    }
}
```

### Low level api

This method is intended to be used when the library does not yet support Twitter's new API.

- You can customize the request yourself.
- You can use `session.send(TwitterAPIRequest,completionHandler:)` to send the request.

```swift

    class YourCustomRequest: TwitterAPIRequest {
        // write code...
    }


    let consumerKey = ""
    let consumerSecret = ""
    let oauthToken = ""
    let oauthTokenSecret = ""

    let client = TwitterAPIKit(
        consumerKey: consumerKey,
        consumerSecret: consumerSecret,
        oauthToken: oauthToken,
        oauthTokenSecret: oauthTokenSecret
    )

    let request = YourCustomRequest()
    client.session.send(request) { result in
        // ...
    }
}
```

### OAuth

#### PIN based

> https://developer.twitter.com/en/docs/authentication/oauth-1-0a/pin-based-oauth

```swift
// for CLI tool
func runOAuthV1() {
    client.v1.postOAuthRequestToken(.init(oauthCallback: "oob")) { result0 in
        do {
            let success = try result0.get()
            print("Token:", success)

            let url = client.v1.makeOAuthAuthorizeURL(.init(oauthToken: success.oauthToken, forceLogin: true))!
            print("Enter this URL into your browser and enter the PIN code that will be displayed after authentication.")
            print(url)

            let pinCode = readLine()!

            client.v1.postOAuthAccessToken(.init(oauthToken: success.oauthToken, oauthVerifier: pinCode)) { result1 in
                do {
                    let success = try result1.get()
                    print("AccessToken:", success)

                } catch let error {
                    print("Error")
                    print(error)
                }
            }
        } catch let error {
            print("Error")
            print(error)
        }
    }
}

// Output of runOAuthV1

/*
 Token: TwitterOAuthTokenV1(oauthToken: "your-token", oauthTokenSecret: "your-secret", oauthCallbackConfirmed: Optional(true))
 Enter this URL into your browser and enter the PIN code that will be displayed after authentication.
 https://api.twitter.com/oauth/authorize?force_login=true&oauth_token=your-token
 > your pin
 AccessToken: TwitterOAuthAccessTokenV1(oauthToken: "", oauthTokenSecret: "", userID: Optional(""), screenName: Optional(""))
*/
```

### App-only authentication and OAuth 2.0 Bearer Token

> https://developer.twitter.com/en/docs/authentication/oauth-2-0/application-only

```swift
func runOAuth2V1() {
    let client = TwitterAPIKit(
        .basic(apiKey: "your consumer key", apiSecretKey: "your consumer secret")
    )

    client.v1.postOAuth2BearerToken(.init()) { result in
        do {
            let success = try result.get()
            print("Token:", success)
        } catch let error {
            print("Error")
            print(error)
        }
    }
}

func useBearerTokenV1() {
    let client = TwitterAPIKit(.bearer("AAAAAAAAAAAAAAAAAAAAA"))
    client.v1.getUserTimeline(.init(target: .screenName("twitterapi"))) { result in
        print(result)
    }
}

```

## TODO

- [ ] Support API v1 endpoint : 70%
- [ ] Support API v2 endpoint
- [ ] Swift Concurrency
- [ ] Document

## Known Issues

When requesting `GET /2/users/:id/tweets` (Twitter API v2) using OAuth 1.0a User Context, the following error occurs when specifying `end_time` or `start_time`, but the cause has not been investigated yet.


```json
{
  "title": "Unauthorized",
  "type": "about:blank",
  "status": 401,
  "detail": "Unauthorized"
}
```
